package de.fraunhofer.iem.dataprovider.taskManager.tasks

import de.fraunhofer.iem.dataprovider.gitlab.GitConfiguration
import de.fraunhofer.iem.dataprovider.taskManager.Event
import de.fraunhofer.iem.dataprovider.taskManager.GetGitlabProjectDone
import org.gitlab4j.api.GitLabApi


sealed class GetGitlabProjectTask : Task() {

    protected abstract val repoId: Any
    protected abstract val gitlabConfiguration: GitConfiguration

    private val gitlabApi: GitLabApi = GitLabApi(gitlabConfiguration.host, gitlabConfiguration.accessToken)

    override suspend fun execute() {
        logger.info("Gitlab config ${gitlabConfiguration.host} ${gitlabConfiguration.accessToken}")

        val project = gitlabApi.projectApi.getProject(repoId)
        logger.info(project.toString())
        val projectUri = project.httpUrlToRepo
        val gitProject = GitProject(project.name, projectUri)

        responseChannel(GetGitlabProjectDone(taskID, gitProject))

        logger.info("Retrieved project ${project.path} and url $projectUri")
    }
}

class GetGitlabProjectPathTask(
    override val repoId: String,
    override val gitlabConfiguration: GitConfiguration,
    override val responseChannel: suspend (task: Event) -> Unit,
) : GetGitlabProjectTask()

class GetGitlabProjectIdTask(
    override val repoId: Long,
    override val gitlabConfiguration: GitConfiguration,
    override val responseChannel: suspend (task: Event) -> Unit
) : GetGitlabProjectTask()
